//
//  Belt.m
//  Packer
//
//  Created by Jason Hurt on 12/14/11.
//  Copyright (c) 2011 UNLV. All rights reserved.
//

#import "Belt.h"
#import "Constants.h"
#import "InventoryItem.h"

@implementation Belt

@synthesize rate, items, lastSpawnTime, nextSpawnDelta;

+ (float)randomFloatBetween:(float)a  and:(float)b {
    float random = ((float) rand()) / (float) RAND_MAX;
    float diff = b - a;
    float r = random * diff;
    return a + r;
}

- (id)initWithX:(float)x andY:(float)y withClipArea:(int)clipArea andSpriteBatchNode:(CCSpriteBatchNode*)spriteBatchNode {
    self = [super initWithFile:@"belt.png"];
    if (self != nil) {
        items = [[NSMutableArray alloc] init];
        _spriteBatchNode=[spriteBatchNode retain];
        nextSpawnDelta=0.1;
        for(int i = 0; i < 10; i++) {
            CCSprite *nextBelt = [CCSprite spriteWithSpriteFrameName:@"belt_single.png"];
            nextBelt.position=ccp(self.contentSize.width*(i+1)/2, self.textureRect.size.height/2);
            [self addChild:nextBelt];
        }
        lastSpawnTime = CACurrentMediaTime();
        float beltWidth = self.textureRect.size.width;
        float beltHeight = self.textureRect.size.height;
        [self setPosition: CGPointMake(x+beltWidth/2, y+beltHeight/2)];
        CCSprite *beltClip = [CCSprite spriteWithSpriteFrameName:[NSString stringWithFormat:@"belt-clip-area-%d.png", clipArea]];
        [beltClip setPosition: CGPointMake(beltClip.textureRect.size.width/2, y+beltClip.textureRect.size.height/2)];    
        [spriteBatchNode addChild:beltClip z:0];
    }
    return self;
}

- (void)startBelt:(float)newRate {
    self.rate=newRate;
    if(self.rate > 0) {
        float time = self.textureRect.size.width/self.rate;
        CCAction *beltAction = [CCRepeatForever actionWithAction: 
                                [CCSequence actions: 
                                 [CCMoveTo actionWithDuration:time position:ccp([self position].x-self.contentSize.width,[self position].y)], 
                                 [CCMoveTo actionWithDuration:0.0f position:ccp([self position].x,[self position].y)], 
                                 nil]];
        [self stopAllActions];
        [self runAction:beltAction];   
    }
}

- (void)newSpawnDelta {
    self.nextSpawnDelta=[Belt randomFloatBetween:MIN_TIME_BETWEEN_SPAWNS and:MAX_TIME_BETWEEN_SPAWNS]/(self.rate/MIN_BELT_SPEED);
}

- (void)update:(ccTime)dt {
    for(InventoryItem *item in self.items) {
        //move any existing items to the left
        if(!item.isFallingOff) {
            float itemDelta = self.rate * dt;
            item.position = ccp(item.position.x-itemDelta,
                                item.position.y); 
            
        }
    }
    
    //check for spawning a new item
    NSTimeInterval currentTime = CACurrentMediaTime();
    double timeDelta = (currentTime - self.lastSpawnTime);
    if([self.items count] < MAX_ITEMS_PER_BELT && timeDelta > self.nextSpawnDelta) {
        InventoryItem *item = [InventoryItem randomInventoryItem];
        item.position=ccp(1000.0 + item.textureRect.size.width*0.5,
                          self.position.y + 10.0);
        
        [_spriteBatchNode addChild:item z:2];
        self.lastSpawnTime = currentTime;
        [self newSpawnDelta];
        [self.items addObject:item];
    }
}

- (void) dealloc {
    [items release];
    [_spriteBatchNode release];
	[super dealloc];
}

@end
