//
//  GameplayLayer.m
//  Packer
//
//  Created by Jason Hurt on 12/13/11.
//  Copyright (c) 2011 UNLV. All rights reserved.
//

#import "GameplayLayer.h"
#import "Constants.h"
#import "Belt.h"
#import "InventoryItem.h"
#import "Score.h"
#import "SimpleAudioEngine.h"

@implementation GameplayLayer

+ (NSString*)getDisplaySeconds:(double)secs {
    secs = MAX(0, secs);
    double intPart = 0;
    double fractPart = modf(secs, &intPart);
    int isecs = (int)intPart;
    int min = isecs / 60;
    int sec = isecs % 60;
    int hund = (int) (fractPart * 100);
    return [NSString stringWithFormat:@"%02d:%02d:%02d", min, sec, hund];
}

- (id)init {
    self = [super init];
    if (self != nil) {
        CGSize screenSize = [[CCDirector sharedDirector] winSize];

        _ensembleItems = [[NSMutableDictionary alloc] init];
        _allEnsembles = [[NSMutableArray alloc] init];
        _totalScore = 0L;

        //setup timer label
        _timeLeftLabel = [[CCLabelTTF labelWithString:[GameplayLayer getDisplaySeconds:GAME_TIME] fontName:@"Minecraftia" fontSize:24] retain];
        [self addChild:_timeLeftLabel z:0];
        _timeLeftLabel.position=ccp(76+screenSize.width/2, 40.0f);
        
        //setup score label
        _scoreLabel = [[CCLabelTTF labelWithString:[NSString stringWithFormat:@"%d", _totalScore] fontName:@"Minecraftia" fontSize:24] retain];
        [self addChild:_scoreLabel z:0];
        _scoreLabel.position=ccp(366.0f, 40.0f);
        
        //setup sprite batch
        [[CCSpriteFrameCache sharedSpriteFrameCache] addSpriteFramesWithFile:@"sprite_game.plist"];
        _spriteBatchNode =[[CCSpriteBatchNode batchNodeWithFile:@"sprite_game.png"] retain];
        [self addChild:_spriteBatchNode z:1];

        //setup moving belts
        _belts = [[NSMutableArray alloc] init];
        float x = 186.0;
        float y = 638.5;
        for(int i = 0; i < 5; i++) {
            Belt *belt = [[Belt alloc] initWithX:x andY:y withClipArea:i+1 andSpriteBatchNode:_spriteBatchNode];
            [self addChild:belt z:0 tag:BELT_TAG];
            int randomBeltSpeed = (arc4random() % MAX_BELT_SPEED)+1;
            if(randomBeltSpeed < MIN_BELT_SPEED) {
                randomBeltSpeed=MIN_BELT_SPEED;
            }
            [belt startBelt:randomBeltSpeed];
            [_belts addObject:belt];
            [belt release];
            y-=122.8;
        }
                
        //setup the guy
        _guy = [[Guy alloc] initWithSpriteFrameName:@"guy_side.png"];
        [_spriteBatchNode addChild:_guy z:3];
        
        //enable touch
        self.isTouchEnabled = YES;
        [[CCTouchDispatcher sharedDispatcher] addTargetedDelegate:self priority:0 swallowsTouches:YES];
        
        //start
        _startTime = CACurrentMediaTime();

        [[SimpleAudioEngine sharedEngine] stopBackgroundMusic];
		[[SimpleAudioEngine sharedEngine] playBackgroundMusic:@"8-bit-happy-nation.mp3" loop:YES];
    }
    return self;
}

-(BOOL)ccTouchBegan:(UITouch *)touch withEvent:(UIEvent *)event {
    CCLOG(@"touch began");
    CGPoint location = [touch locationInView:[touch view]];
    location = [[CCDirector sharedDirector] convertToGL:location];
    if(CGRectContainsPoint(_guy.boundingBox, location)) {
        _guy.isBeingDragged=YES;
    }
    return YES; 
}

- (void)ccTouchEnded:(UITouch *)touch withEvent:(UIEvent *)event {
	CCLOG(@"touch ended");
    CGPoint location = [touch locationInView:[touch view]];
	CCLOG(@"%@: %f, %f", @"touch point", location.x, location.y);
    location = [[CCDirector sharedDirector] convertToGL:location];
    if(CGRectContainsPoint(_guy.guyMoveAreaRect, location) || _guy.isBeingDragged) {
        _guy.lastTouchGuyY = 123.0f;
        //see which belt the touch is nearest to
        if(location.y > 638.0f) {
            _guy.lastTouchGuyY = 638.0f;
        }
        else if(location.y > 515.0f) {
            _guy.lastTouchGuyY = 515.0f;
        }
        else if(location.y > 392.0f) {
            _guy.lastTouchGuyY = 392.0f;
        }
        else if(location.y > 269.0f) {
            _guy.lastTouchGuyY = 269.0f;
        }
    }
    _guy.isBeingDragged=NO;
}

- (void)ccTouchMoved:(UITouch *)touch withEvent:(UIEvent *)event {     
    CCLOG(@"touch moved");
    if(_guy.isBeingDragged) {
        CGPoint location = [self convertTouchToNodeSpace:touch];
        _guy.lastTouchGuyY = location.y;
    }
}

- (void)itemDidFallOffBelt:(id)sender data:(void*)data {
    InventoryItem *item = (InventoryItem*)sender;
    if(item.isFallingOff) {
        Belt *belt = (Belt*)data;
        [belt.items removeObject:sender];
        [_spriteBatchNode removeChild:sender cleanup:YES];
    }
}

- (void)itemsDidGetScored:(id)sender data:(void*)data {
    NSNumber *score = (NSNumber*) data;
    _totalScore += [score intValue];
    _scoreLabel.string=[NSString stringWithFormat:@"%d", _totalScore];
}

- (void)itemDidGetScored:(id)sender {
    [_spriteBatchNode removeChild:sender cleanup:YES];
}

- (void)update:(ccTime)dt {
    float timeDelta = CACurrentMediaTime()-_startTime;
    float timeLeft = GAME_TIME - timeDelta;
    _timeLeftLabel.string=[GameplayLayer getDisplaySeconds:timeLeft];

    //check if the game is over
    if(timeDelta >= GAME_TIME) {
        [[SimpleAudioEngine sharedEngine] stopBackgroundMusic];
        Score *score = [[Score alloc] initWithEnsembles:_allEnsembles];
        CCScene *scene = [CCScene node];
        [scene addChild:score];
        [self removeAllChildrenWithCleanup:YES];
        [[CCDirector sharedDirector] pushScene:scene];
        [score release];
    }
    
    [_guy update:dt];

    //handle the clothes
    for(Belt *belt in _belts) {
        [belt update:dt];

        NSMutableArray *itemsToRemove = [[NSMutableArray alloc] init];
        for(InventoryItem *item in belt.items) {            
            //check for items touching guy or items falling off the belt
            if(CGRectIntersectsRect([_guy adjustedBoundingBox], item.boundingBox)) {
                [item showChildren];
                //guy touched item, give item to guy
                if(item.type==INVENTORY_ITEM_TYPE_TOP) {
                    InventoryItem *existingItem = [_ensembleItems objectForKey:@"top"];
                    if(existingItem) {
                        [_spriteBatchNode removeChild:existingItem cleanup:YES];
                    }
                    [_ensembleItems setObject:item forKey:@"top"];
                    [item makeSmall];
                }
                else if(item.type==INVENTORY_ITEM_TYPE_BOTTOM) {
                    InventoryItem *existingItem = [_ensembleItems objectForKey:@"bottom"];
                    if(existingItem) {
                        [_spriteBatchNode removeChild:existingItem cleanup:YES];
                    }
                    [_ensembleItems setObject:item forKey:@"bottom"];
                    [item makeSmall];
                }
                else if(item.type==INVENTORY_ITEM_TYPE_SHOE) {
                    InventoryItem *existingItem = [_ensembleItems objectForKey:@"shoe"];
                    if(existingItem) {
                        [_spriteBatchNode removeChild:existingItem cleanup:YES];
                    }
                    [_ensembleItems setObject:item forKey:@"shoe"];
                    [item makeSmall];
                }
                
                [item stopAllActions];
                item.opacity=255;
                [itemsToRemove addObject:item];
            }
            else if(!item.isFallingOff && CGRectIntersectsRect(_guy.guyMoveAreaRect, item.boundingBox)) {
                //item is at the end of the belt, fade it out
                CCFadeOut *fadeOut = [[CCFadeOut alloc] initWithDuration:0.8f];
                id callback = [CCCallFuncND actionWithTarget:self selector:@selector(itemDidFallOffBelt:data:) data:belt];
                [item runAction:[CCSequence actions: fadeOut, callback, nil]];
                [fadeOut release];
                [item hideChildren];
                item.isFallingOff=YES;
            }
        }
        
        for(InventoryItem *item in itemsToRemove) {
            [belt.items removeObject:item];
        }
        [itemsToRemove release];
        
    }
    
    //check for a full ensemble
    if([_ensembleItems count] == 3) {
        //blink the items
        for(InventoryItem *item in [_ensembleItems allValues]) {
            CCBlink *blink = [[CCBlink alloc] initWithDuration:1.5f blinks:5];
            id callback = [CCCallFuncN actionWithTarget:self selector:@selector(itemDidGetScored:)];
            [item runAction:[CCSequence actions: blink, callback, nil]];
            [blink release];
        }

        //calculate the new score
        int score = [InventoryItem calculateScore:_ensembleItems];
        CCDelayTime *delay = [[CCDelayTime alloc] initWithDuration:1.5f];
        id callback = [CCCallFuncND actionWithTarget:self selector:@selector(itemsDidGetScored:data:) data:[[NSNumber numberWithInt:score] retain]];
        [self runAction:[CCSequence actions:delay, callback, nil]];
        [delay release];
        
        //keep for next scene
        [_allEnsembles addObject:_ensembleItems];
        
        //reset the inventory
        [_ensembleItems release];
        _ensembleItems = [[NSMutableDictionary alloc] init];
    }
    
    [super update:dt];
}

- (void) dealloc {
    [_guy release];
    [_belts release];
    [_spriteBatchNode release];
    [_timeLeftLabel release];
    [_ensembleItems release];
    [_allEnsembles release];
    [super dealloc];
}

@end
