//
//  InventoryItem.m
//  Packer
//
//  Created by Jason Hurt on 12/15/11.
//  Copyright (c) 2011 UNLV. All rights reserved.
//

#import "InventoryItem.h"
#import "Constants.h"

@implementation InventoryItem

@synthesize type, invColor, gender, brand, brandName, smallImage, largeImage, isFallingOff, brandImage;

- (id)initWithSpriteFrameName:(NSString *)spriteFrameName {
    self = [super initWithSpriteFrameName:spriteFrameName];
    if(self) {
        self.isFallingOff=NO;
        self.largeImage=spriteFrameName;
    }
    return self;
}

- (void)hideChildren {
    for(id child in self.children) {
        CCSprite *sprite = (CCSprite*)child;
        sprite.opacity=0;
    }
}

- (void)showChildren {
    for(id child in self.children) {
        CCSprite *sprite = (CCSprite*)child;
        sprite.opacity=255;
    }
}

- (void)makeSmall {
    if(self.type==INVENTORY_ITEM_TYPE_TOP) {
        [self setDisplayFrame:[[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:self.smallImage]];
        self.position=ccp(SMALL_INVENTORY_TOP_X+self.textureRect.size.width*0.5, SMALL_INVENTORY_Y+2+self.textureRect.size.height*0.5);
    }
    else if(self.type==INVENTORY_ITEM_TYPE_BOTTOM){
        [self setDisplayFrame:[[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:self.smallImage]];
        self.position=ccp(SMALL_INVENTORY_BOTTOM_X+self.textureRect.size.width*0.5, SMALL_INVENTORY_Y+2+self.textureRect.size.height*0.5);
        
    }
    else if(self.type==INVENTORY_ITEM_TYPE_SHOE) {
        [self setDisplayFrame:[[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:self.smallImage]];
        self.position=ccp(SMALL_INVENTORY_SHOES_X+self.textureRect.size.width*0.5, SMALL_INVENTORY_Y+2+self.textureRect.size.height*0.5);
    }
    CCSprite *brandSprite = (CCSprite*)[self getChildByTag:BRAND_SPRITE_TAG];
    if(brandSprite) {
        brandSprite.scale = 0.5f;
        brandSprite.position = ccp(self.textureRect.size.width/2, self.textureRect.size.height/2);
    }
    CCSprite *genderSprite = (CCSprite*)[self getChildByTag:GENDER_SPRITE_TAG];
    if(genderSprite) {
        genderSprite.scale=0.8f;
        genderSprite.position=ccp(-7.5f, self.textureRect.size.height-10.0f);
    }
}

- (void)makeBig {
    [self setDisplayFrame:[[CCSpriteFrameCache sharedSpriteFrameCache] spriteFrameByName:self.largeImage]];
    CCSprite *brandSprite = (CCSprite*)[self getChildByTag:BRAND_SPRITE_TAG];
    if(brandSprite) {
        brandSprite.scale = 0.6f;
        brandSprite.position = ccp(self.textureRect.size.width/2, self.textureRect.size.height/2);
    }
    CCSprite *genderSprite = (CCSprite*)[self getChildByTag:GENDER_SPRITE_TAG];
    if(genderSprite) {
        genderSprite.scale = 1.0f;
        genderSprite.position=ccp(-7.5f, self.textureRect.size.height-10.0f);
    }

}

+ (int)newRandom:(int)max {
    return 1 + (arc4random() % max);
}

+ (int)calculateScore:(NSDictionary*)ensemble {
    int score = 0;
    InventoryItem *top = [ensemble objectForKey:@"top"];
    InventoryItem *bottom = [ensemble objectForKey:@"bottom"];
    InventoryItem *shoe = [ensemble objectForKey:@"shoe"];
    //match on all things
    if(top.invColor==bottom.invColor==shoe.invColor) {
        score += 10000;
    }
    if(top.gender==bottom.gender==shoe.gender) {
        score += 10000;
    }
    if(top.brand==bottom.brand==shoe.brand) {
        score += 10000;
    }
    if(score == 20000) {
        score += 30000;
    }
    else if(score==30000) {
        score +=70000;
    }
    
    BOOL matchedColor=NO;
    BOOL matchedGender=NO;
    BOOL matchedBrand=NO;
    //match on two things
    if(((top.invColor==bottom.invColor) ||
       (top.invColor==shoe.invColor) ||
        (shoe.invColor==bottom.invColor)) && !(top.invColor==bottom.invColor==shoe.invColor)) {
        score += 2500;
        matchedColor=YES;
    }
    if(((top.gender==bottom.gender) ||
        (top.gender==shoe.gender) ||
        (shoe.gender==bottom.gender)) && !(top.gender==bottom.gender==shoe.gender)) {
        matchedGender=YES;
    }
    if(((top.brand==bottom.brand) ||
        (top.brand==shoe.brand) ||
        (shoe.brand==bottom.brand)) && !(top.brand==bottom.brand==shoe.brand)) {
        score += 2500;
        matchedBrand=YES;
    }
    if(matchedBrand&&matchedColor&&matchedGender) {
        score += 15000;
    }
    else if(matchedBrand &&matchedColor) {
        score+=5000;
    }
    else if(matchedBrand &&matchedGender) {
        score+=5000;
    }
    else if(matchedColor &&matchedGender) {
        score+=5000;
    }
    
    if(score == 0) {
        //player is a walking retard
        score = 100;
    }
    return score;
}

+ (InventoryItem*) randomInventoryItem {
    int type = 0;
    NSString *typeName=nil;
    int randType = [InventoryItem newRandom:INVENTORY_ITEM_TYPE_SHOE];
    if(randType <= INVENTORY_ITEM_TYPE_TOP) {
        type = INVENTORY_ITEM_TYPE_TOP;
        typeName = @"shirt";
    }
    else if(randType <= INVENTORY_ITEM_TYPE_BOTTOM) {
        type = INVENTORY_ITEM_TYPE_BOTTOM;
        typeName = @"pants";
    }
    else if(randType <= INVENTORY_ITEM_TYPE_SHOE) {
        type = INVENTORY_ITEM_TYPE_SHOE;
        typeName = @"shoe";
    }
    else if(randType <= INVENTORY_ITEM_TYPE_ACCESSORY) {
        type = INVENTORY_ITEM_TYPE_ACCESSORY;
    }
    
    int invColor = 0;
    NSString *colorName=nil;
    int randColor = [InventoryItem newRandom:INV_COLOR_YELLOW];
    if(randColor<=INV_COLOR_BLUE) {
        invColor = INV_COLOR_BLUE;
        colorName=@"blue";
    }
    else if(randColor<=INV_COLOR_GREEN) {
        invColor = INV_COLOR_GREEN;
        colorName=@"green";
    }
    else if(randColor<=INV_COLOR_GREY) {
        invColor = INV_COLOR_GREY;
        colorName=@"grey";
    }
    else if(randColor<=INV_COLOR_ORANGE) {
        invColor = INV_COLOR_ORANGE;
        colorName=@"orange";
    }
    else if(randColor<=INV_COLOR_PINK) {
        invColor = INV_COLOR_PINK;
        colorName=@"pink";
    }
    else if(randColor<=INV_COLOR_PURPLE) {
        invColor = INV_COLOR_PURPLE;
        colorName=@"purple";
    }
    else if(randColor<=INV_COLOR_RED) {
        invColor = INV_COLOR_RED;
        colorName=@"red";
    }
    else if(randColor<=INV_COLOR_TURQUOISE) {
        invColor = INV_COLOR_TURQUOISE;
        colorName=@"turquoise";
    }
    else if(randColor<=INV_COLOR_YELLOW) {
        invColor = INV_COLOR_YELLOW;
        colorName=@"yellow";
    }
    
    InventoryItem *item = [[InventoryItem alloc] initWithSpriteFrameName:[NSString stringWithFormat:@"%@_%@.png", typeName, colorName]];
    item.smallImage=[NSString stringWithFormat:@"%@_%@_small.png", typeName, colorName];
    item.invColor=invColor;
    item.type=type;
    
    int randGender = [InventoryItem newRandom:INV_GENDER_FEMALE];
    if(randGender <= INV_GENDER_MALE) {
        item.gender = INV_GENDER_MALE;
    }
    else if(randGender <= INV_GENDER_FEMALE) {
        item.gender = INV_GENDER_FEMALE;
    }
    
    int randBrand = [InventoryItem newRandom:BRAND_UGG];
    if(randBrand <= BRAND_ECCO) {
        item.brand=BRAND_ECCO;
        item.brandName=@"ECCO";
        item.brandImage=@"brand_ecco.png";
    }
    else if(randBrand <= BRAND_NIKE) {
        item.brand=BRAND_NIKE;
        item.brandName=@"NIKE";
        item.brandImage=@"brand_nike.png";
    }
    else if(randBrand <= BRAND_PUMA) {
        item.brand=BRAND_PUMA;
        item.brandName=@"PUMA";
        item.brandImage=@"brand_puma.png";
    }
    else if(randBrand <= BRAND_UGG) {
        item.brand=BRAND_UGG;
        item.brandName=@"UGG";
        item.brandImage=@"brand_ugg.png";
    }
    
    //add the gender sprite
    CCSprite *genderSprite;
    if(item.gender==INV_GENDER_MALE) {
        genderSprite = [CCSprite spriteWithSpriteFrameName:@"gender_male.png"];
    }
    else {
        genderSprite = [CCSprite spriteWithSpriteFrameName:@"gender_female.png"];
    }
    genderSprite.position=ccp(-15.0f, item.textureRect.size.height-20.0f);
    [item addChild:genderSprite z:1 tag:GENDER_SPRITE_TAG];
    
    //add the brand sprite
    CCSprite *brand = [CCSprite spriteWithSpriteFrameName:item.brandImage];
    brand.scale=0.75f;
    brand.position = ccp(item.textureRect.size.width/2, item.textureRect.size.height/2);
    
    [item addChild:brand z:2 tag:BRAND_SPRITE_TAG];

    
    return [item autorelease];
}


- (void) dealloc {
 	[super dealloc];
}


@end
