//
//  Score.m
//  Home
//
//  Created by Jennifer Wilhelm on 12/14/11.
//  Copyright 2011 Zappos. All rights reserved.
//

#import "Score.h"
#import "SimpleAudioEngine.h"
#import "InventoryItem.h"
#import "GameScene.h"

@implementation Score

+ (CCScene*)scene {
    CCScene *scene = [CCScene node];
    [scene addChild:[Score node]];
    return scene;
}

- (void)tallyScore:(id)sender {
    
    // Total Score label and dropshadow
    CCLabelTTF *label = [CCLabelTTF labelWithString:@"total score" fontName:@"Minecraftia" fontSize:28];
    CCLabelTTF *labelDropShadow = [CCLabelTTF labelWithString:@"total score" fontName:@"Minecraftia" fontSize:28];
    label.color = ccc3(231, 166, 11);
    
    // Total Score position
    if ([_ensembles count] % 3 != 0) {
        NSLog(@"Not mod 3");
        label.position = ccp(_ss.width/2, _startY-130);
        labelDropShadow.position = ccp(label.position.x+1, label.position.y+1);
    } else {
        NSLog(@"mod 3");
        label.position = ccp(_ss.width/2, _startY+55);
        labelDropShadow.position = ccp(label.position.x+1, label.position.y+1);
    }
    
    labelDropShadow.color = ccc3(1, 55, 66);
    [self addChild:labelDropShadow];
    [self addChild:label];
    
    // Format score
    NSNumber* number = [NSNumber numberWithInt:_totalScore];
    NSNumberFormatter *numberFormatter = [[NSNumberFormatter alloc] init];
    [numberFormatter setNumberStyle:kCFNumberFormatterDecimalStyle];
    [numberFormatter setGroupingSeparator:@","];
    NSString* commaString = [numberFormatter stringForObjectValue:number];
    [numberFormatter release];
    
    // Calculate total score
    NSString *scoreString = [NSString stringWithFormat:@"%@", commaString];
    int positionIncrement = 0;
    
    for (int i = 0; i < [scoreString length]; i++) {
        unichar numChar = [scoreString characterAtIndex:i];
        NSString *currentNumber = [NSString stringWithCharacters:&numChar length:1];
        NSString *filename;
        bool isComma = false;
        if([currentNumber isEqualToString:@","]) {
            filename = @"gameover_comma.png"; 
            isComma = true;
        } else {
            filename = [NSString stringWithFormat:@"gameover_%d.png", [currentNumber intValue]]; 
        }
        
        CCSprite *numberImage = [CCSprite spriteWithSpriteFrameName:filename];
        int numberImageWidth = 80;
        
        // Find starting X position (this is probably super ghetto, I dunno how else to do it)
        int scoreWidth = ([scoreString length] * numberImageWidth) / 2 - 40;
        
        if(isComma) {
            numberImage.position = ccp(_ss.width/2 - scoreWidth + positionIncrement, label.position.y - numberImageWidth - 30);
        }
        else {
            numberImage.position = ccp(_ss.width/2 - scoreWidth + positionIncrement, label.position.y - numberImageWidth);
        }
        
        [self addChild:numberImage];
        positionIncrement += numberImageWidth;
    }
    
    // Play Total Score sound http://www.premiumbeat.com/sfx/detail/accent-twinkle
    [[SimpleAudioEngine sharedEngine] playEffect:@"high_score.mp3"];
    
    _finished=YES;
}

- (void)scoreDidShow:(id)sender data:(void*)data {
    NSNumber *index = (NSNumber*)data;
    NSDictionary *ensemble = [_ensembles objectAtIndex:[index intValue]];
    CCSprite *ensembleFrame =[_ensembleFrames objectAtIndex:[index intValue]];
    CCFadeIn *fadeInFrame = [[CCFadeIn alloc] initWithDuration:0.5f];
    
    // Add ensemble frame
    [self removeChild:ensembleFrame cleanup:NO];
    [self addChild:ensembleFrame];
    [ensembleFrame setPosition:ccp(_startX, _startY)];
    ensembleFrame.opacity=0;
    
    // Format score
    int score = [InventoryItem calculateScore:ensemble];
    NSNumber* number = [NSNumber numberWithInt:score];
    NSNumberFormatter *numberFormatter = [[NSNumberFormatter alloc] init];
    [numberFormatter setNumberStyle:kCFNumberFormatterDecimalStyle];
    [numberFormatter setGroupingSeparator:@","];
    NSString* commaString = [numberFormatter stringForObjectValue:number];
    [numberFormatter release];
    
    // Add score below frame
    NSString *scoreString = [NSString stringWithFormat:@"+%@", commaString]; 
    CCLabelTTF *scoreLabel = [CCLabelTTF labelWithString:scoreString fontName:@"Minecraftia" fontSize:22];
    CCLabelTTF *scoreLabelDropShadow = [CCLabelTTF labelWithString:scoreString fontName:@"Minecraftia" fontSize:22];
    CCFadeIn *fadeInScoreLabel = [[[CCFadeIn alloc] initWithDuration:0.5f] autorelease];
    CCFadeIn *fadeInScoreLabelDropShadow = [[[CCFadeIn alloc] initWithDuration:0.5f] autorelease];
    scoreLabel.position = ccp(_startX, _startY - ensembleFrame.textureRect.size.height/2 - 15);
    scoreLabel.color = ccc3(231, 166, 11);
    scoreLabel.opacity = 0;
    scoreLabelDropShadow.position = ccp(scoreLabel.position.x+1, scoreLabel.position.y+1);
    scoreLabelDropShadow.color = ccc3(1, 55, 66);
    scoreLabelDropShadow.opacity = 0;
    [self addChild:scoreLabelDropShadow];
    [self addChild:scoreLabel];
    [scoreLabel runAction:fadeInScoreLabelDropShadow];
    [scoreLabel runAction:fadeInScoreLabel];
    
    // Add clothes to frame
    InventoryItem *top = [ensemble objectForKey:@"top"];
    [top makeBig];
    top.position = ccp(ensembleFrame.position.x - 80, ensembleFrame.position.y);
                     
    InventoryItem *bottom = [ensemble objectForKey:@"bottom"];
    [bottom makeBig];
    bottom.position = ccp(ensembleFrame.position.x, ensembleFrame.position.y);

    InventoryItem *shoe = [ensemble objectForKey:@"shoe"];
    [shoe makeBig];
    shoe.position = ccp(ensembleFrame.position.x + 80, ensembleFrame.position.y);
    
    [self addChild:top];
    [self addChild:bottom];
    [self addChild:shoe];
    
    // Play score sound http://www.premiumbeat.com/sfx/detail/accent-twinkle
    [[SimpleAudioEngine sharedEngine] playEffect:@"add_score_loop.mp3"];
    
    // Calculate score
    _totalScore += score;
    
    // Update position of next frame
    if (([index intValue]+1) % 3 == 0) {
        _startY = _startY - 200;
        _startX = 170;
    } 
    else {
        _startX = _startX + 345;
    }
    // If we're done, trigger the Tally Score callback
    if ([index intValue]+1 == [_ensembles count]) {
        CCDelayTime *delayTime = [CCDelayTime actionWithDuration:.25];
        id endCallback = [CCCallFuncN actionWithTarget:self selector:@selector(tallyScore:)];
        [ensembleFrame runAction:[CCSequence actions:fadeInFrame, delayTime, endCallback, nil]];
        //[delayTime release];
    }
    else {
        id callback = [CCCallFuncND actionWithTarget:self selector:@selector(scoreDidShow:data:) data:[NSNumber numberWithInt:([index intValue]+1)]];
        [ensembleFrame runAction:[CCSequence actions:fadeInFrame, callback, nil]];
    }
    [fadeInFrame release];
}

- (id)initWithEnsembles:(NSArray*)ensembles {
    self = [super init];
    if (self) {
        _finished = NO;
        _ensembles = [ensembles retain];
        _ensembleFrames = [[NSMutableArray alloc] init];
        _gameSpriteBatchNode = [CCSpriteBatchNode batchNodeWithFile:@"sprite_game.png"];
        _ss = [[CCDirector sharedDirector] winSize];
        _startX = 170;
        _startY = _ss.height - 100;
        
        // Add sprite
        [[CCSpriteFrameCache sharedSpriteFrameCache] addSpriteFramesWithFile:@"sprite_game.plist"];
        [self addChild:_gameSpriteBatchNode];
        
        // Background and rotation
        CCSprite *homeBackground = [CCSprite spriteWithFile:@"sunburst.png"];
        id action = [CCRepeatForever actionWithAction: [CCRotateBy actionWithDuration:50 angle:360]];
        [homeBackground setPosition: CGPointMake(_ss.width/2, _ss.height/2)];
        [homeBackground runAction:action];
        [self addChild:homeBackground z:0 tag:0];
        
        _totalScore = 0;
        
        if([_ensembles count] > 0) {
            for(int i = 0; i < [_ensembles count]; i++) {
                CCSprite *ensembleFrame = [CCSprite spriteWithSpriteFrameName:@"ensemble_frame.png"];
                [_ensembleFrames addObject:ensembleFrame];
                if(i==0) {
                    // Animate ensemble frames and score
                    [self scoreDidShow:ensembleFrame data:[NSNumber numberWithInt:i]];
                }
            }
        }
        else {
            CCLabelTTF *suckAssLabel = [CCLabelTTF labelWithString:@"Sorry, You Suck Ass!" fontName:@"Minecraftia" fontSize:24];
            suckAssLabel.position=ccp(_ss.width/2,_ss.height/2);
            [self addChild:suckAssLabel];
            _finished=YES;
        }
        
        //enable touch
        self.isTouchEnabled = YES;
        [[CCTouchDispatcher sharedDispatcher] addTargetedDelegate:self priority:0 swallowsTouches:YES];
    }
    
    return self;
}

-(BOOL)ccTouchBegan:(UITouch *)touch withEvent:(UIEvent *)event {
    return _finished;
}

- (void)ccTouchEnded:(UITouch *)touch withEvent:(UIEvent *)event {
    [[CCDirector sharedDirector] popScene];
    [[CCDirector sharedDirector] popScene];
    [[CCDirector sharedDirector] pushScene:[GameScene node]];
}

- (void) dealloc {
    [_ensembles release];
    [_ensembleFrames release];
	[super dealloc];
}


@end